/******************************************************************
Created with PROGRAMINO IDE for Arduino
Project     : WS2813 smart RGB LEDs
Board       : MAKERFACTORY EBB ESP32
Version     : 1.0
Date        : 28.12.2018
Author      : MF

Description : WS2813 smart RGB-LEDs demo

Modified Adafruit example

Switches:
RGB LED SW312 ON

******************************************************************/

#include <Adafruit_NeoPixel.h> // https://github.com/adafruit/Adafruit_NeoPixel

// LED data pin
#define PIN 2

// How many NeoPixels are attached to the Arduino?
#define NUMPIXELS 4

// Parameter 1 = number of pixels in strip
// Parameter 2 = Arduino pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
//   NEO_RGBW    Pixels are wired for RGBW bitstream (NeoPixel RGBW products)
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

// IMPORTANT: To reduce NeoPixel burnout risk, add 1000 uF capacitor across
// pixel power leads, add 300 - 500 Ohm resistor on first pixel's data input
// and minimize distance between Arduino and first pixel.  Avoid connecting
// on a live circuit...if you must, connect GND first.

void setup() 
{
    pixels.begin(); // This initializes the NeoPixel library
    pixels.show();  // Initialize all pixels to 'off'
}

void loop() 
{
    // Some example procedures showing how to display to the pixels:
    colorWipe(pixels.Color(255, 0, 0), 50); // Red
    colorWipe(pixels.Color(0, 255, 0), 50); // Green
    colorWipe(pixels.Color(0, 0, 255), 50); // Blue
    
    //colorWipe(strip.Color(0, 0, 0, 255), 50); // White RGBW
    // Send a theater pixel chase in...
    theaterChase(pixels.Color(127, 127, 127), 50);   // White
    theaterChase(pixels.Color(127, 0, 0), 50);       // Red
    theaterChase(pixels.Color(0, 0, 127), 50);       // Blue
    
    rainbow(20);
    rainbowCycle(20);
    theaterChaseRainbow(50);
}

// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint8_t wait) 
{
    for(uint16_t i=0; i<pixels.numPixels(); i++) 
    {
        pixels.setPixelColor(i, c);
        pixels.show();
        delay(wait);
    }
}

void rainbow(uint8_t wait) 
{
    uint16_t i, j;
    
    for(j=0; j<256; j++) 
    {
        for(i=0; i<pixels.numPixels(); i++) 
        {
            pixels.setPixelColor(i, Wheel((i+j) & 255));
        }
        pixels.show();
        delay(wait);
    }
}

// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait) 
{
    uint16_t i, j;
    
    for(j=0; j<256*5; j++) // 5 cycles of all colors on wheel
    { 
        for(i=0; i< pixels.numPixels(); i++) 
        {
            pixels.setPixelColor(i, Wheel(((i * 256 / pixels.numPixels()) + j) & 255));
        }
        pixels.show();
        delay(wait);
    }
}

//Theatre-style crawling lights.
void theaterChase(uint32_t c, uint8_t wait) 
{
    for (int j=0; j<10; j++) //do 10 cycles of chasing
    {  
        for (int q=0; q < 3; q++) 
        {
            for (uint16_t i=0; i < pixels.numPixels(); i=i+3) 
            {
                pixels.setPixelColor(i+q, c); //turn every third pixel on   
            }
            pixels.show();
            delay(wait);
            
            for (uint16_t i=0; i < pixels.numPixels(); i=i+3) 
            {
                pixels.setPixelColor(i+q, 0); //turn every third pixel off
            }
        }
    }
}

//Theatre-style crawling lights with rainbow effect
void theaterChaseRainbow(uint8_t wait) 
{
    for (int j=0; j < 256; j++) // cycle all 256 colors in the wheel
    {     
        for (int q=0; q < 3; q++) 
        {
            for (uint16_t i=0; i < pixels.numPixels(); i=i+3) 
            {
                pixels.setPixelColor(i+q, Wheel( (i+j) % 255)); //turn every third pixel on
            }
            pixels.show();
            delay(wait);
            
            for (uint16_t i=0; i < pixels.numPixels(); i=i+3) 
            {
                pixels.setPixelColor(i+q, 0); //turn every third pixel off
            }
        }
    }
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) 
{
    WheelPos = 255 - WheelPos;
    if(WheelPos < 85) 
    {
        return pixels.Color(255 - WheelPos * 3, 0, WheelPos * 3);
    }
    if(WheelPos < 170) 
    {
        WheelPos -= 85;
        return pixels.Color(0, WheelPos * 3, 255 - WheelPos * 3);
    }
    WheelPos -= 170;
    return pixels.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}
